DRAGON_ELEMENTS_TYPES = {
    "ph": "pysical", "w": "sea", "p": "nature","f": "flame","d": "dark","e": "terra",
    "el": "electric", "m": "metal","i": "ice","wr": "war",
    "l": "legend", "li": "light", "pu": "pure","bt": "beauty",
    "ch": "chaos", "mg": "magic", "hp": "happy","dr": "dream",
    "so": "soul", "pr": "primal", "wd": "wind","ti": "time"
}

__all__ = [
    "DRAGON_ELEMENTS_TYPES"
]